# """ Defines the parsers needed to parse ofproto flows
# """
#
# import functools
# import re
#
# from ovs_dbg.kv import KVParser, KVDecoders, ParseError
# from ovs_dbg.list import ListParser, ListDecoders
# from ovs_dbg.decoders import (decode_int, decode_mask32, decode_default,
#                              nested_kv_decoder, kv_decoder,
#                              nested_list_decoder)
#
# class OFPFlow:
#    """OpenFlow Flow"""
#
#    def __init__(self, data, actions):
#        """Constructor"""
#        self._data = data
#        self._actions = actions
#
#    def actions(self):
#        return self._actions
#
#    def keys(self):
#        return self._data
#
#    @classmethod
#    def from_string(cls, ofp_string):
#        """Parse a ofproto flow string
#
#        :param ofp_string: a ofproto string as dumped by ovs-ofctl tool
#        :type ofp_string: str
#
#        :return: an OFPFlow with the content of the flow string
#        :rtype: OFPFlow
#        """
#        parts = ofp_string.split("actions=")
#        if len(parts) != 2:
#            raise ValueError("malformed ofproto flow: {}", ofp_string)
#
#        match_decoders = cls._match_decoders()
#        mdecoder = KVParser(match_decoders)
#        mdecoder.parse(parts[0])
#
#        act_decoders = cls._act_decoders()
#        adecoder = KVParser(act_decoders)
#        adecoder.parse(parts[1])
#
#        return cls(mdecoder.kv(), adecoder.kv())
#
#    @classmethod
#    def _match_decoders(cls):
#        """Generate the match decoders"""
#        regs = {"reg%s" % i: decode_mask32 for i in range(0, 16)}
#        matches = {
#            "priority": decode_int,
#            "metadata": decode_int,
#        }
#        return KVDecoders(decoders={**matches, **regs})
#
#    @classmethod
#    def _act_decoders(cls):
#        """Generate the actions decoders"""
#        adec = dict()
#
#        return KVDecoders(adec,
#                          default_free=decode_free_action_field)
#
#    def __str__(self):
#        string = "Matches: "
#        for match in self._data:
#            string += "    " + str(match)
#        string += "\n"
#
#        string += "Actions: "
#        for act in self._actions:
#            string += "    " + str(act)
#
#        return string
#
#
# def decode_free_action_field(value):
#    """decoder that handles free (non key-value) vields in action strings
#
#    :param: value: the free value to decode
#    :type value: str
#
#    :return: the key and the value to store
#    :rtype: (string, [int or string])
#    """
#    try:
#        return "output", int(value)
#    except ValueError:
#        return "output", value
#
#
##
##def decode_controller(value):
##    """decoder for controller action
##    """
##    if not value:
##        return True
##    else:
##        # Try controller:max_len
##        try:
##            max_len = int(value)
##            return {
##                "max_len": max_len,
##            }
##        except ValueError:
##            pass
##        # controller(key[=val], ...)
##        return nested_kv_decoder()(value)
##
##
##def decode_field_action(value):
##    """decoder for ofproto field modification actions
##
##    Example:
##        load:0x001122334455->OXM_OF_ETH_SRC[]
##        move:reg0[0..5]->reg1[26..31]
##    """
##    parts = value.split("->")
##    if len(parts) < 2:
##        raise ParseError("Invalid field action: %s" % value)
##    return {
##        "source": parts[0],
##        "destination": parts[1],
##    }
##
##
##def decode_enqueue(value):
##    """decoder that handles free (non key-value) vields in action strings
##
##    :param: value: the free value to decode
##    :type value: str
##
##    :return: the key and the value to store
##    :rtype: (string, [int or string])
##    """
##    parts = re.split(r',\:', value)
##    if len(parts) != 2:
##        raise ParseError("malformed enqueue value: %s" & value)
##    try:
##        port = int(parts[0])
##    except ValueError:
##        port = parts[0]
##
##    try:
##        queue = int(parts[1], 0)
##    except ValueError:
##        raise ParseError("malformed enqueue value: %s" & value)
##
##    return {
##        "port": port,
##        "queue": queue,
##    }
##
##def decode_free_action_field(value):
##    """decoder that handles free (non key-value) vields in action strings
##
##    :param: value: the free value to decode
##    :type value: str
##
##    :return: the key and the value to store
##    :rtype: (string, [int or string])
##    """
##    try:
##        return "output", int(value)
##    except ValueError:
##        return "output", value
##



""" Defines helpful decoders that can be used to decode information from the
flows

A decoder is generally a callable that accepts a string and returns the value
object
"""


def decode_default(value):
    """Default decoder.

    It tries to convert into an integer value and, if it fails, just
    returns the string.
    """
    try:
        ival = int(value, 0)
        return ival
    except ValueError:
        return value



# This file is auto-generated. Do not edit

import functools
from ovs_dbg import decoders

field_decoders = {
    "dp_hash": functools.partial(decoders.decode_mask, 32),
    "recirc_id": decoders.decode_int,
    "packet_type": decoders.decode_default,
    "conj_id": decoders.decode_int,
    "tun_id": functools.partial(decoders.decode_mask, 64),
    "tunnel_id": functools.partial(decoders.decode_mask, 64),
    "tun_src": decoders.decode_ip,
    "tun_dst": decoders.decode_ip,
    "tun_ipv6_src": decoders.decode_ip,
    "tun_ipv6_dst": decoders.decode_ip,
    "tun_flags": decoders.decode_default,
    "tun_ttl": decoders.decode_int,
    "tun_tos": decoders.decode_int,
    "tun_gbp_id": functools.partial(decoders.decode_mask, 16),
    "tun_gbp_flags": functools.partial(decoders.decode_mask, 8),
    "tun_erspan_idx": functools.partial(decoders.decode_mask, 20),
    "tun_erspan_ver": functools.partial(decoders.decode_mask, 4),
    "tun_erspan_dir": functools.partial(decoders.decode_mask, 1),
    "tun_erspan_hwid": functools.partial(decoders.decode_mask, 6),
    "tun_gtpu_flags": functools.partial(decoders.decode_mask, 8),
    "tun_gtpu_msgtype": functools.partial(decoders.decode_mask, 8),
    "tun_metadata0": functools.partial(decoders.decode_mask, 992),
    "tun_metadata1": functools.partial(decoders.decode_mask, 992),
    "tun_metadata2": functools.partial(decoders.decode_mask, 992),
    "tun_metadata3": functools.partial(decoders.decode_mask, 992),
    "tun_metadata4": functools.partial(decoders.decode_mask, 992),
    "tun_metadata5": functools.partial(decoders.decode_mask, 992),
    "tun_metadata6": functools.partial(decoders.decode_mask, 992),
    "tun_metadata7": functools.partial(decoders.decode_mask, 992),
    "tun_metadata8": functools.partial(decoders.decode_mask, 992),
    "tun_metadata9": functools.partial(decoders.decode_mask, 992),
    "tun_metadata10": functools.partial(decoders.decode_mask, 992),
    "tun_metadata11": functools.partial(decoders.decode_mask, 992),
    "tun_metadata12": functools.partial(decoders.decode_mask, 992),
    "tun_metadata13": functools.partial(decoders.decode_mask, 992),
    "tun_metadata14": functools.partial(decoders.decode_mask, 992),
    "tun_metadata15": functools.partial(decoders.decode_mask, 992),
    "tun_metadata16": functools.partial(decoders.decode_mask, 992),
    "tun_metadata17": functools.partial(decoders.decode_mask, 992),
    "tun_metadata18": functools.partial(decoders.decode_mask, 992),
    "tun_metadata19": functools.partial(decoders.decode_mask, 992),
    "tun_metadata20": functools.partial(decoders.decode_mask, 992),
    "tun_metadata21": functools.partial(decoders.decode_mask, 992),
    "tun_metadata22": functools.partial(decoders.decode_mask, 992),
    "tun_metadata23": functools.partial(decoders.decode_mask, 992),
    "tun_metadata24": functools.partial(decoders.decode_mask, 992),
    "tun_metadata25": functools.partial(decoders.decode_mask, 992),
    "tun_metadata26": functools.partial(decoders.decode_mask, 992),
    "tun_metadata27": functools.partial(decoders.decode_mask, 992),
    "tun_metadata28": functools.partial(decoders.decode_mask, 992),
    "tun_metadata29": functools.partial(decoders.decode_mask, 992),
    "tun_metadata30": functools.partial(decoders.decode_mask, 992),
    "tun_metadata31": functools.partial(decoders.decode_mask, 992),
    "tun_metadata32": functools.partial(decoders.decode_mask, 992),
    "tun_metadata33": functools.partial(decoders.decode_mask, 992),
    "tun_metadata34": functools.partial(decoders.decode_mask, 992),
    "tun_metadata35": functools.partial(decoders.decode_mask, 992),
    "tun_metadata36": functools.partial(decoders.decode_mask, 992),
    "tun_metadata37": functools.partial(decoders.decode_mask, 992),
    "tun_metadata38": functools.partial(decoders.decode_mask, 992),
    "tun_metadata39": functools.partial(decoders.decode_mask, 992),
    "tun_metadata40": functools.partial(decoders.decode_mask, 992),
    "tun_metadata41": functools.partial(decoders.decode_mask, 992),
    "tun_metadata42": functools.partial(decoders.decode_mask, 992),
    "tun_metadata43": functools.partial(decoders.decode_mask, 992),
    "tun_metadata44": functools.partial(decoders.decode_mask, 992),
    "tun_metadata45": functools.partial(decoders.decode_mask, 992),
    "tun_metadata46": functools.partial(decoders.decode_mask, 992),
    "tun_metadata47": functools.partial(decoders.decode_mask, 992),
    "tun_metadata48": functools.partial(decoders.decode_mask, 992),
    "tun_metadata49": functools.partial(decoders.decode_mask, 992),
    "tun_metadata50": functools.partial(decoders.decode_mask, 992),
    "tun_metadata51": functools.partial(decoders.decode_mask, 992),
    "tun_metadata52": functools.partial(decoders.decode_mask, 992),
    "tun_metadata53": functools.partial(decoders.decode_mask, 992),
    "tun_metadata54": functools.partial(decoders.decode_mask, 992),
    "tun_metadata55": functools.partial(decoders.decode_mask, 992),
    "tun_metadata56": functools.partial(decoders.decode_mask, 992),
    "tun_metadata57": functools.partial(decoders.decode_mask, 992),
    "tun_metadata58": functools.partial(decoders.decode_mask, 992),
    "tun_metadata59": functools.partial(decoders.decode_mask, 992),
    "tun_metadata60": functools.partial(decoders.decode_mask, 992),
    "tun_metadata61": functools.partial(decoders.decode_mask, 992),
    "tun_metadata62": functools.partial(decoders.decode_mask, 992),
    "tun_metadata63": functools.partial(decoders.decode_mask, 992),
    "metadata": functools.partial(decoders.decode_mask, 64),
    "in_port": decoders.decode_default,
    "in_port_oxm": decoders.decode_default,
    "actset_output": decoders.decode_default,
    "skb_priority": decoders.decode_int,
    "pkt_mark": functools.partial(decoders.decode_mask, 32),
    "ct_state": decoders.decode_default,
    "ct_zone": decoders.decode_int,
    "ct_mark": functools.partial(decoders.decode_mask, 32),
    "ct_label": functools.partial(decoders.decode_mask, 128),
    "ct_nw_proto": decoders.decode_int,
    "ct_nw_src": decoders.decode_ip,
    "ct_nw_dst": decoders.decode_ip,
    "ct_ipv6_src": decoders.decode_ip,
    "ct_ipv6_dst": decoders.decode_ip,
    "ct_tp_src": functools.partial(decoders.decode_mask, 16),
    "ct_tp_dst": functools.partial(decoders.decode_mask, 16),
    "reg0": functools.partial(decoders.decode_mask, 32),
    "reg1": functools.partial(decoders.decode_mask, 32),
    "reg2": functools.partial(decoders.decode_mask, 32),
    "reg3": functools.partial(decoders.decode_mask, 32),
    "reg4": functools.partial(decoders.decode_mask, 32),
    "reg5": functools.partial(decoders.decode_mask, 32),
    "reg6": functools.partial(decoders.decode_mask, 32),
    "reg7": functools.partial(decoders.decode_mask, 32),
    "reg8": functools.partial(decoders.decode_mask, 32),
    "reg9": functools.partial(decoders.decode_mask, 32),
    "reg10": functools.partial(decoders.decode_mask, 32),
    "reg11": functools.partial(decoders.decode_mask, 32),
    "reg12": functools.partial(decoders.decode_mask, 32),
    "reg13": functools.partial(decoders.decode_mask, 32),
    "reg14": functools.partial(decoders.decode_mask, 32),
    "reg15": functools.partial(decoders.decode_mask, 32),
    "xreg0": functools.partial(decoders.decode_mask, 64),
    "xreg1": functools.partial(decoders.decode_mask, 64),
    "xreg2": functools.partial(decoders.decode_mask, 64),
    "xreg3": functools.partial(decoders.decode_mask, 64),
    "xreg4": functools.partial(decoders.decode_mask, 64),
    "xreg5": functools.partial(decoders.decode_mask, 64),
    "xreg6": functools.partial(decoders.decode_mask, 64),
    "xreg7": functools.partial(decoders.decode_mask, 64),
    "xxreg0": functools.partial(decoders.decode_mask, 128),
    "xxreg1": functools.partial(decoders.decode_mask, 128),
    "xxreg2": functools.partial(decoders.decode_mask, 128),
    "xxreg3": functools.partial(decoders.decode_mask, 128),
    "eth_src": decoders.decode_mac,
    "dl_src": decoders.decode_mac,
    "eth_dst": decoders.decode_mac,
    "dl_dst": decoders.decode_mac,
    "eth_type": decoders.decode_int,
    "dl_type": decoders.decode_int,
    "vlan_tci": functools.partial(decoders.decode_mask, 16),
    "dl_vlan": decoders.decode_int,
    "vlan_vid": functools.partial(decoders.decode_mask, 12),
    "dl_vlan_pcp": decoders.decode_int,
    "vlan_pcp": decoders.decode_int,
    "mpls_label": decoders.decode_int,
    "mpls_tc": decoders.decode_int,
    "mpls_bos": decoders.decode_int,
    "mpls_ttl": decoders.decode_int,
    "ip_src": decoders.decode_ip,
    "nw_src": decoders.decode_ip,
    "ip_dst": decoders.decode_ip,
    "nw_dst": decoders.decode_ip,
    "ipv6_src": decoders.decode_ip,
    "ipv6_dst": decoders.decode_ip,
    "ipv6_label": functools.partial(decoders.decode_mask, 20),
    "nw_proto": decoders.decode_int,
    "ip_proto": decoders.decode_int,
    "nw_tos": decoders.decode_int,
    "ip_dscp": decoders.decode_int,
    "nw_ecn": decoders.decode_int,
    "ip_ecn": decoders.decode_int,
    "nw_ttl": decoders.decode_int,
    "ip_frag": decoders.decode_default,
    "nw_frag": decoders.decode_default,
    "arp_op": decoders.decode_int,
    "arp_spa": decoders.decode_ip,
    "arp_tpa": decoders.decode_ip,
    "arp_sha": decoders.decode_mac,
    "arp_tha": decoders.decode_mac,
    "tcp_src": functools.partial(decoders.decode_mask, 16),
    "tp_src": functools.partial(decoders.decode_mask, 16),
    "tcp_dst": functools.partial(decoders.decode_mask, 16),
    "tp_dst": functools.partial(decoders.decode_mask, 16),
    "tcp_flags": decoders.decode_default,
    "udp_src": functools.partial(decoders.decode_mask, 16),
    "udp_dst": functools.partial(decoders.decode_mask, 16),
    "sctp_src": functools.partial(decoders.decode_mask, 16),
    "sctp_dst": functools.partial(decoders.decode_mask, 16),
    "icmp_type": decoders.decode_int,
    "icmp_code": decoders.decode_int,
    "icmpv6_type": decoders.decode_int,
    "icmpv6_code": decoders.decode_int,
    "nd_target": decoders.decode_ip,
    "nd_sll": decoders.decode_mac,
    "nd_tll": decoders.decode_mac,
    "nd_reserved": decoders.decode_int,
    "nd_options_type": decoders.decode_int,
    "nsh_flags": functools.partial(decoders.decode_mask, 8),
    "nsh_mdtype": decoders.decode_int,
    "nsh_np": decoders.decode_int,
    "nsh_spi": decoders.decode_int,
    "nsp": decoders.decode_int,
    "nsh_si": decoders.decode_int,
    "nsi": decoders.decode_int,
    "nsh_c1": functools.partial(decoders.decode_mask, 32),
    "nshc1": functools.partial(decoders.decode_mask, 32),
    "nsh_c2": functools.partial(decoders.decode_mask, 32),
    "nshc2": functools.partial(decoders.decode_mask, 32),
    "nsh_c3": functools.partial(decoders.decode_mask, 32),
    "nshc3": functools.partial(decoders.decode_mask, 32),
    "nsh_c4": functools.partial(decoders.decode_mask, 32),
    "nshc4": functools.partial(decoders.decode_mask, 32),
    "nsh_ttl": decoders.decode_int,
}



""" Defines the parsers needed to parse ofproto flows
"""

from flows import CSLParser


def kv_time(_, value):
    """time key-value parser

    Used for fields such as:
        duration=1234.123s
    """
    time_str = value.rstrip("s")
    return float(time_str)


def kv_int(_, value):
    """integer parser

    Used for fields such as:
        n_bytes=34
    """
    return int(value)



""" Defines helpful decoders that can be used to decode information from the
flows

A decoder is generally a callable that accepts a string and
returns the value object
"""

import functools
import netaddr

from ovs_dbg.kv import KVParser
from ovs_dbg.list import ListParser


def decode_default(value):
    """Default decoder.

    It tries to convert into an integer value and, if it fails, just
    returns the string.
    """
    try:
        ival = int(value, 0)
        return ival
    except ValueError:
        return value


def decode_int(value):
    """integer decoder

    Both base10 and base16 integers are supported

    Used for fields such as:
        n_bytes=34
        metadata=0x4
    """
    return int(value, 0)


def decode_time(value):
    """time decoder

    Used for fields such as:
        duration=1234.123s
    """
    time_str = value.rstrip("s")
    return float(time_str)


def decode_mask(size, value):
    """value/mask decoder for values of specific size (bits)

    Used for fields such as:
        reg0=0x248/0xff
    """
    parts = value.split("/")
    if len(parts) > 1:
        value = int(parts[0], 0)
        mask = int(parts[1], 0)
    else:
        value = int(parts[0], 0)
        mask = 2 ** size - 1

    return {"value": value, "mask": mask}


decode_mask8 = functools.partial(decode_mask, 8)
decode_mask16 = functools.partial(decode_mask, 16)
decode_mask32 = functools.partial(decode_mask, 32)
decode_mask64 = functools.partial(decode_mask, 64)


def decode_ip(value):
    """IP address decoder. Supports both IPv4 and IPv6 addresses

    Used for fields such as:
        nw_src=192.168.1.1
        nw_src=192.168.1.0/24
        nw_src=192.168.1.0/255.255.255.0
        nw_dst=2001:db8::1000
        nw_dst=2001:db8::0/24
    """
    try:
        ipnet = netaddr.IPNetwork(value)
        return ipnet
    except netaddr.AddrFormatError:
        raise ValueError("value %s: is not an ipv4 or ipv6 address" % value)


def decode_mac(value):
    """MAC address decoder"""
    try:
        mac = netaddr.EUI(value)
        return mac
    except netaddr.AddrFormatError:
        raise ValueError("value %s: is not an mac address" % value)


def decode_nested_kv(decoders, value):
    """A key-value decoder that extracts nested key-value pairs and returns
    them in a dictionary

    Args:
        decoders (KVDecoders): the KVDecoders to use.
        value (str): the value string to decode.
    """
    parser = KVParser(decoders)
    parser.parse(value)
    return {kv.key: kv.value for kv in parser.kv()}


def nested_kv_decoder(decoders=None):
    """Helper function that creates a nested kv decoder with given KVDecoders"""
    return functools.partial(decode_nested_kv, decoders)


def decode_nested_list(decoders, delims, value):
    """Extracts nested list from te string and returns it in a dictionary
    them in a dictionary

    Args:
        decoders (ListDecoders): the ListDecoders to use.
        value (str): the value string to decode.
    """
    parser = ListParser(decoders, delims)
    parser.parse(value)
    return {kv.key: kv.value for kv in parser.kv()}


def nested_list_decoder(decoders=None, delims=None):
    """Helper function that creates a nested list decoder with given
    ListDecoders
    """
    return functools.partial(decode_nested_list, decoders, delims)


def kv_decoder(decoder):
    """Convert simpler value decoder into kv_decoder

    A kv_decoder is a callable that accepts a keyword and a value string
    and returns the key and value objects. In most cases, the returned keyword
    is the same as the one provided.
    This helper function converts a value decoder (that just takes the value
    string as parameter and returns an object) into a kv_decoder where the key
    is the same
    """

    def kv_decoder(key, value):
        return key, decoder(value)

    return kv_decoder



""" Defines helpful decoders that can be used to decode information from the
flows

A decoder is generally a callable that accepts a string and returns the value
object
"""

import functools
import netaddr


def decode_default(value):
    """Default decoder.

    It tries to convert into an integer value and, if it fails, just
    returns the string.
    """
    try:
        ival = int(value, 0)
        return ival
    except ValueError:
        return value


def decode_flag(value):
    """Default a flag. It's exising is just flagged by returning True"""
    return True


def decode_int(value):
    """integer decoder

    Both base10 and base16 integers are supported

    Used for fields such as:
        n_bytes=34
        metadata=0x4
    """
    return int(value, 0)


def decode_time(value):
    """time decoder

    Used for fields such as:
        duration=1234.123s
    """
    time_str = value.rstrip("s")
    return float(time_str)


def decode_mask(size, value):
    """value/mask decoder for values of specific size (bits)

    Used for fields such as:
        reg0=0x248/0xff
    """
    parts = value.split("/")
    if len(parts) > 1:
        value = int(parts[0], 0)
        mask = int(parts[1], 0)
    else:
        value = int(parts[0], 0)
        mask = 2 ** size - 1

    return {"value": value, "mask": mask}


decode_mask8 = functools.partial(decode_mask, 8)
decode_mask16 = functools.partial(decode_mask, 16)
decode_mask32 = functools.partial(decode_mask, 32)
decode_mask64 = functools.partial(decode_mask, 64)
decode_mask128 = functools.partial(decode_mask, 128)


class EthMask:
    """EthMask represents an Ethernet address with optional mask

    It uses netaddr.EUI

    Attributes:
        eth (netaddr.EUI): the ethernet address
        mask (netaddr.EUI): Optional, the ethernet address mask

    Args:
        string (str): A string representing the masked ethernet address
            e.g: 00.11:22:33:44:55 or 01:00:22:00:33:00/01:00:00:00:00:00
    """

    def __init__(self, string):
        mask_parts = string.split("/")
        self._eth = netaddr.EUI(mask_parts[0])
        if len(mask_parts) == 2:
            self._mask = netaddr.EUI(mask_parts[1])
        else:
            self._mask = None

    @property
    def eth(self):
        """The ethernet address"""
        return self._eth

    @property
    def mask(self):
        """The ethernet address mask"""
        return self._mask

    def __eq__(self, other):
        """Returns True if this EthMask is numerically the same as other"""
        return self._mask == other._mask and self._eth == other._eth

    def __contains__(self, other):
        """
        Args:
            other (netaddr.EUI): another Ethernet address

        Returns:
            True if other falls into the masked address range
        """
        if self._mask:
            return (other.value & self._mask.value) == (
                self._eth.value & self._mask.value
            )
        else:
            return other == self._eth

    def __str__(self):
        if self._mask:
            return "/".join(
                [
                    self._eth.format(netaddr.mac_unix),
                    self._mask.format(netaddr.mac_unix),
                ]
            )
        else:
            return self._eth.format(netaddr.mac_unix)

    def __repr__(self):
        return "%s('%s')" % (self.__class__.__name__, self)


def decode_mac(value):
    """MAC address decoder"""
    return EthMask(value)


class IPMask:
    """IPMask stores an IPv6 or IPv4 and a mask

    It uses netaddr.IPAddress. The IPMask can be represented by a
    netaddr.IPNetwork (if it's a valid cidr) or by an ip and a mask

    Args:
        string (str): A string representing the ip/mask
    """

    def __init__(self, string):
        """Constructor"""
        self._ipnet = None
        self._ip = None
        self._mask = None
        try:
            self._ipnet = netaddr.IPNetwork(string)
        except netaddr.AddrFormatError:
            pass

        if not self._ipnet:
            # It's not a valid CIDR. Store ip and mask indendently
            parts = string.split("/")
            if len(parts) != 2:
                raise ValueError(
                    "value {}: is not an ipv4 or ipv6 address".format(string)
                )
            try:
                self._ip = netaddr.IPAddress(parts[0])
                self._mask = netaddr.IPAddress(parts[1])
            except netaddr.AddrFormatError as exc:
                raise ValueError(
                    "value {}: is not an ipv4 or ipv6 address".format(string)
                ) from exc

    def __eq__(self, other):
        """Returns True if this IPMask is numerically the same as other"""
        if self._ipnet:
            return self._ipnet == other._ipnet

        return self._ip == other._ip and self._mask == other._mask

    def __contains__(self, other):
        """
        Args:
            other (netaddr.IPAddres): another IP address

        Returns:
            True if other falls into the masked ip range
        """
        if self._ipnet:
            return other in self._ipnet
        return (other & self._mask) == (self._ip & self._mask)

    def cidr(self):
        """
        Returns True if the IPMask is a valid CIDR
        """
        return self._ipnet is not None

    @property
    def ip(self):
        """The IP address"""
        if self._ipnet:
            return self._ipnet.ip
        return self._ip

    @property
    def mask(self):
        """The IP mask"""
        if self._ipnet:
            return self._ipnet.netmask
        return self._mask

    def __str__(self):
        if self._ipnet:
            return str(self._ipnet)
        return "/".join([str(self._ip), str(self._mask)])

    def __repr__(self):
        return "%s('%s')" % (self.__class__.__name__, self)


def decode_ip(value):
    """IP address decoder. Supports both IPv4 and IPv6 addresses

    Used for fields such as:
        nw_src=192.168.1.1
        nw_src=192.168.1.0/24
        nw_src=192.168.1.0/255.255.255.0
        nw_dst=2001:db8::1000
        nw_dst=2001:db8::0/24
    """
    return IPMask(value)
